@@ -128,6 +128,14 @@ describe("vector", function()
 		assert.equal(vector.new(4.1, 5.9, 5.5), a:apply(f))
 	end)
 
+	it("combine()", function()
+		local a = vector.new(1, 2, 3)
+		local b = vector.new(3, 2, 1)
+		assert.equal(vector.add(a, b), vector.combine(a, b, function(x, y) return x + y end))
+		assert.equal(vector.new(3, 2, 3), vector.combine(a, b, math.max))
+		assert.equal(vector.new(1, 2, 1), vector.combine(a, b, math.min))
+	end)
+
 	it("equals()", function()
 		local function assertE(a, b)
 			assert.is_true(vector.equals(a, b))
