@@ -78,54 +78,63 @@ local function load_texture_packs(txtpath, retval)
 
 	for _, item in ipairs(list) do
 		if item ~= "base" then
-			local name = item
-
 			local path = txtpath .. DIR_DELIM .. item .. DIR_DELIM
-			if path == current_texture_path then
-				name = fgettext("$1 (Enabled)", name)
-			end
-
 			local conf = Settings(path .. "texture_pack.conf")
+			local enabled = path == current_texture_path
 
+			local title = conf:get("title") or item
+
+			-- list_* is only used if non-nil, else the regular versions are used.
 			retval[#retval + 1] = {
 				name = item,
+				title = title,
+				list_name = enabled and fgettext("$1 (Enabled)", item) or nil,
+				list_title = enabled and fgettext("$1 (Enabled)", title) or nil,
 				author = conf:get("author"),
 				release = tonumber(conf:get("release")) or 0,
-				list_name = name,
 				type = "txp",
 				path = path,
-				enabled = path == current_texture_path,
+				enabled = enabled,
 			}
 		end
 	end
 end
 
-function get_mods(path,retval,modpack)
+--modmanager implementation
+pkgmgr = {}
+
+--- Scans a directory recursively for mods and adds them to `listing`
+-- @param path         Absolute directory path to scan recursively
+-- @param virtual_path Prettified unique path (e.g. "mods", "mods/mt_modpack")
+-- @param listing      Input. Flat array to insert located mods and modpacks
+-- @param modpack      Currently processing modpack or nil/"" if none (recursion)
+function pkgmgr.get_mods(path, virtual_path, listing, modpack)
 	local mods = core.get_dir_list(path, true)
 
 	for _, name in ipairs(mods) do
 		if name:sub(1, 1) ~= "." then
-			local prefix = path .. DIR_DELIM .. name
+			local mod_path = path .. DIR_DELIM .. name
+			local mod_virtual_path = virtual_path .. "/" .. name
 			local toadd = {
 				dir_name = name,
 				parent_dir = path,
 			}
-			retval[#retval + 1] = toadd
+			listing[#listing + 1] = toadd
 
 			-- Get config file
 			local mod_conf
-			local modpack_conf = io.open(prefix .. DIR_DELIM .. "modpack.conf")
+			local modpack_conf = io.open(mod_path .. DIR_DELIM .. "modpack.conf")
 			if modpack_conf then
 				toadd.is_modpack = true
 				modpack_conf:close()
 
-				mod_conf = Settings(prefix .. DIR_DELIM .. "modpack.conf"):to_table()
+				mod_conf = Settings(mod_path .. DIR_DELIM .. "modpack.conf"):to_table()
 				if mod_conf.name then
 					name = mod_conf.name
 					toadd.is_name_explicit = true
 				end
 			else
-				mod_conf = Settings(prefix .. DIR_DELIM .. "mod.conf"):to_table()
+				mod_conf = Settings(mod_path .. DIR_DELIM .. "mod.conf"):to_table()
 				if mod_conf.name then
 					name = mod_conf.name
 					toadd.is_name_explicit = true
@@ -134,14 +143,16 @@ function get_mods(path,retval,modpack)
 
 			-- Read from config
 			toadd.name = name
+			toadd.title = mod_conf.title
 			toadd.author = mod_conf.author
 			toadd.release = tonumber(mod_conf.release) or 0
-			toadd.path = prefix
+			toadd.path = mod_path
+			toadd.virtual_path = mod_virtual_path
 			toadd.type = "mod"
 
 			-- Check modpack.txt
 			-- Note: modpack.conf is already checked above
-			local modpackfile = io.open(prefix .. DIR_DELIM .. "modpack.txt")
+			local modpackfile = io.open(mod_path .. DIR_DELIM .. "modpack.txt")
 			if modpackfile then
 				modpackfile:close()
 				toadd.is_modpack = true
@@ -153,14 +164,18 @@ function get_mods(path,retval,modpack)
 			elseif toadd.is_modpack then
 				toadd.type = "modpack"
 				toadd.is_modpack = true
-				get_mods(prefix, retval, name)
+				pkgmgr.get_mods(mod_path, mod_virtual_path, listing, name)
 			end
 		end
 	end
-end
 
---modmanager implementation
-pkgmgr = {}
+	if not modpack then
+		-- Sort all when the recursion is done
+		table.sort(listing, function(a, b)
+			return a.virtual_path:lower() < b.virtual_path:lower()
+		end)
+	end
+end
 
 function pkgmgr.get_texture_packs()
 	local txtpath = core.get_texturepath()
@@ -174,7 +189,7 @@ function pkgmgr.get_texture_packs()
 	end
 
 	table.sort(retval, function(a, b)
-		return a.name > b.name
+		return a.title:lower() < b.title:lower()
 	end)
 
 	return retval
@@ -240,101 +255,12 @@ function pkgmgr.get_base_folder(temppath)
 end
 
 --------------------------------------------------------------------------------
-function pkgmgr.isValidModname(modpath)
-	if modpath:find("-") ~= nil then
-		return false
-	end
-
-	return true
+function pkgmgr.is_valid_modname(modpath)
+	return modpath:match("[^a-z0-9_]") == nil
 end
 
 --------------------------------------------------------------------------------
-function pkgmgr.parse_register_line(line)
-	local pos1 = line:find("\"")
-	local pos2 = nil
-	if pos1 ~= nil then
-		pos2 = line:find("\"",pos1+1)
-	end
-
-	if pos1 ~= nil and pos2 ~= nil then
-		local item = line:sub(pos1+1,pos2-1)
-
-		if item ~= nil and
-			item ~= "" then
-			local pos3 = item:find(":")
-
-			if pos3 ~= nil then
-				local retval = item:sub(1,pos3-1)
-				if retval ~= nil and
-					retval ~= "" then
-					return retval
-				end
-			end
-		end
-	end
-	return nil
-end
-
---------------------------------------------------------------------------------
-function pkgmgr.parse_dofile_line(modpath,line)
-	local pos1 = line:find("\"")
-	local pos2 = nil
-	if pos1 ~= nil then
-		pos2 = line:find("\"",pos1+1)
-	end
-
-	if pos1 ~= nil and pos2 ~= nil then
-		local filename = line:sub(pos1+1,pos2-1)
-
-		if filename ~= nil and
-			filename ~= "" and
-			filename:find(".lua") then
-			return pkgmgr.identify_modname(modpath,filename)
-		end
-	end
-	return nil
-end
-
---------------------------------------------------------------------------------
-function pkgmgr.identify_modname(modpath,filename)
-	local testfile = io.open(modpath .. DIR_DELIM .. filename,"r")
-	if testfile ~= nil then
-		local line = testfile:read()
-
-		while line~= nil do
-			local modname = nil
-
-			if line:find("minetest.register_tool") then
-				modname = pkgmgr.parse_register_line(line)
-			end
-
-			if line:find("minetest.register_craftitem") then
-				modname = pkgmgr.parse_register_line(line)
-			end
-
-
-			if line:find("minetest.register_node") then
-				modname = pkgmgr.parse_register_line(line)
-			end
-
-			if line:find("dofile") then
-				modname = pkgmgr.parse_dofile_line(modpath,line)
-			end
-
-			if modname ~= nil then
-				testfile:close()
-				return modname
-			end
-
-			line = testfile:read()
-		end
-		testfile:close()
-	end
-
-	return nil
-end
---------------------------------------------------------------------------------
-function pkgmgr.render_packagelist(render_list)
+function pkgmgr.render_packagelist(render_list, use_technical_names, with_error)
 	if not render_list then
 		if not pkgmgr.global_mods then
 			pkgmgr.refresh_globals()
@@ -346,31 +272,75 @@ function pkgmgr.render_packagelist(render_list)
 	local retval = {}
 	for i, v in ipairs(list) do
 		local color = ""
+		local icon = 0
+		local error = with_error and with_error[v.virtual_path]
+		local function update_error(val)
+			if val and (not error or (error.type == "warning" and val.type == "error")) then
+				error = val
+			end
+		end
+
 		if v.is_modpack then
 			local rawlist = render_list:get_raw_list()
 			color = mt_color_dark_green
 
-			for j = 1, #rawlist, 1 do
-				if rawlist[j].modpack == list[i].name and
-						not rawlist[j].enabled then
-					-- Modpack not entirely enabled so showing as grey
-					color = mt_color_grey
-					break
+			for j = 1, #rawlist do
+				if rawlist[j].modpack == list[i].name then
+					if with_error then
+						update_error(with_error[rawlist[j].virtual_path])
+					end
+
+					if rawlist[j].enabled then
+						icon = 1
+					else
+						-- Modpack not entirely enabled so showing as grey
+						color = mt_color_grey
+					end
 				end
 			end
 		elseif v.is_game_content or v.type == "game" then
+			icon = 1
 			color = mt_color_blue
+
+			local rawlist = render_list:get_raw_list()
+			if v.type == "game" and with_error then
+				for j = 1, #rawlist do
+					if rawlist[j].is_game_content then
+						update_error(with_error[rawlist[j].virtual_path])
+					end
+				end
+			end
 		elseif v.enabled or v.type == "txp" then
+			icon = 1
 			color = mt_color_green
 		end
 
+		if error then
+			if error.type == "warning" then
+				color = mt_color_orange
+				icon = 2
+			else
+				color = mt_color_red
+				icon = 3
+			end
+		end
+
 		retval[#retval + 1] = color
 		if v.modpack ~= nil or v.loc == "game" then
 			retval[#retval + 1] = "1"
 		else
 			retval[#retval + 1] = "0"
 		end
-		retval[#retval + 1] = core.formspec_escape(v.list_name or v.name)
+
+		if with_error then
+			retval[#retval + 1] = icon
+		end
+
+		if use_technical_names then
+			retval[#retval + 1] = core.formspec_escape(v.list_name or v.name)
+		else
+			retval[#retval + 1] = core.formspec_escape(v.list_title or v.list_name or v.title or v.name)
+		end
 	end
 
 	return table.concat(retval, ",")
@@ -397,6 +367,14 @@ function pkgmgr.is_modpack_entirely_enabled(data, name)
 	return true
 end
 
+local function disable_all_by_name(list, name, except)
+	for i=1, #list do
+		if list[i].name == name and list[i] ~= except then
+			list[i].enabled = false
+		end
+	end
+end
+
 ---------- toggles or en/disables a mod or modpack and its dependencies --------
 local function toggle_mod_or_modpack(list, toggled_mods, enabled_mods, toset, mod)
 	if not mod.is_modpack then
@@ -405,13 +383,16 @@ local function toggle_mod_or_modpack(list, toggled_mods, enabled_mods, toset, mo
 			toset = not mod.enabled
 		end
 		if mod.enabled ~= toset then
-			mod.enabled = toset
 			toggled_mods[#toggled_mods+1] = mod.name
 		end
 		if toset then
 			-- Mark this mod for recursive dependency traversal
 			enabled_mods[mod.name] = true
+
+			-- Disable other mods with the same name
+			disable_all_by_name(list, mod.name, mod)
 		end
+		mod.enabled = toset
 	else
 		-- Toggle or en/disable every mod in the modpack,
 		-- interleaved unsupported
@@ -472,6 +453,7 @@ function pkgmgr.enable_mod(this, toset)
 			end
 		end
 	end
+
 	-- If sp is 0, every dependency is already activated
 	while sp > 0 do
 		local name = to_enable[sp]
@@ -483,8 +465,8 @@ function pkgmgr.enable_mod(this, toset)
 			if not mod_to_enable then
 				core.log("warning", "Mod dependency \"" .. name ..
 					"\" not found!")
-			else
-				if not mod_to_enable.enabled  then
+			elseif not mod_to_enable.is_game_content then
+				if not mod_to_enable.enabled then
 					mod_to_enable.enabled = true
 					toggled_mods[#toggled_mods+1] = mod_to_enable.name
 				end
@@ -538,22 +520,28 @@ function pkgmgr.get_worldconfig(worldpath)
 end
 
 --------------------------------------------------------------------------------
-function pkgmgr.install_dir(type, path, basename, targetpath)
+function pkgmgr.install_dir(expected_type, path, basename, targetpath)
+	assert(type(expected_type) == "string")
+	assert(type(path) == "string")
+	assert(basename == nil or type(basename) == "string")
+	assert(targetpath == nil or type(targetpath) == "string")
+
 	local basefolder = pkgmgr.get_base_folder(path)
 
-	-- There's no good way to detect a texture pack, so let's just assume
-	-- it's correct for now.
-	if type == "txp" then
+	if expected_type == "txp" then
+		assert(basename)
+
+		-- There's no good way to detect a texture pack, so let's just assume
+		-- it's correct for now.
 		if basefolder and basefolder.type ~= "invalid" and basefolder.type ~= "txp" then
 			return nil, fgettext("Unable to install a $1 as a texture pack", basefolder.type)
 		end
 
 		local from = basefolder and basefolder.path or path
-		if targetpath then
-			core.delete_dir(targetpath)
-		else
+		if not targetpath then
 			targetpath = core.get_texturepath() .. DIR_DELIM .. basename
 		end
+		core.delete_dir(targetpath)
 		if not core.copy_dir(from, targetpath, false) then
 			return nil,
 				fgettext("Failed to install $1 to $2", basename, targetpath)
@@ -561,74 +549,38 @@ function pkgmgr.install_dir(type, path, basename, targetpath)
 		return targetpath, nil
 
 	elseif not basefolder then
-		return nil, fgettext("Unable to find a valid mod or modpack")
+		return nil, fgettext("Unable to find a valid mod, modpack, or game")
 	end
 
-	--
-	-- Get destination
-	--
-	if basefolder.type == "modpack" then
-		if type ~= "mod" then
-			return nil, fgettext("Unable to install a modpack as a $1", type)
-		end
+	-- Check type
+	if basefolder.type ~= expected_type and (basefolder.type ~= "modpack" or expected_type ~= "mod") then
+		return nil, fgettext("Unable to install a $1 as a $2", basefolder.type, expected_type)
+	end
 
-		-- Get destination name for modpack
-		if targetpath then
-			core.delete_dir(targetpath)
+	-- Set targetpath if not predetermined
+	if not targetpath then
+		local content_path
+		if basefolder.type == "modpack" or basefolder.type == "mod" then
+			if not basename then
+				basename = get_last_folder(cleanup_path(basefolder.path))
+			end
+			content_path = core.get_modpath()
+		elseif basefolder.type == "game" then
+			content_path = core.get_gamepath()
 		else
-			local clean_path = nil
-			if basename ~= nil then
-				clean_path = basename
-			end
-			if not clean_path then
-				clean_path = get_last_folder(cleanup_path(basefolder.path))
-			end
-			if clean_path then
-				targetpath = core.get_modpath() .. DIR_DELIM .. clean_path
-			else
-				return nil,
-					fgettext("Install Mod: Unable to find suitable folder name for modpack $1",
-					path)
-			end
-		end
-	elseif basefolder.type == "mod" then
-		if type ~= "mod" then
-			return nil, fgettext("Unable to install a mod as a $1", type)
+			error("Unknown content type")
 		end
 
-		if targetpath then
-			core.delete_dir(targetpath)
+		if basename and (basefolder.type ~= "mod" or pkgmgr.is_valid_modname(basename)) then
+			targetpath = content_path .. DIR_DELIM .. basename
 		else
-			local targetfolder = basename
-			if targetfolder == nil then
-				targetfolder = pkgmgr.identify_modname(basefolder.path, "init.lua")
-			end
-
-			-- If heuristic failed try to use current foldername
-			if targetfolder == nil then
-				targetfolder = get_last_folder(basefolder.path)
-			end
-
-			if targetfolder ~= nil and pkgmgr.isValidModname(targetfolder) then
-				targetpath = core.get_modpath() .. DIR_DELIM .. targetfolder
-			else
-				return nil, fgettext("Install Mod: Unable to find real mod name for: $1", path)
-			end
-		end
-
-	elseif basefolder.type == "game" then
-		if type ~= "game" then
-			return nil, fgettext("Unable to install a game as a $1", type)
-		end
-
-		if targetpath then
-			core.delete_dir(targetpath)
-		else
-			targetpath = core.get_gamepath() .. DIR_DELIM .. basename
+			return nil,
+				fgettext("Install: Unable to find suitable folder name for $1", path)
 		end
 	end
 
 	-- Copy it
+	core.delete_dir(targetpath)
 	if not core.copy_dir(basefolder.path, targetpath, false) then
 		return nil,
 			fgettext("Failed to install $1 to $2", basename, targetpath)
@@ -652,13 +604,14 @@ function pkgmgr.preparemodlist(data)
 
 	--read global mods
 	local modpaths = core.get_modpaths()
-	for _, modpath in ipairs(modpaths) do
-		get_mods(modpath, global_mods)
+	for key, modpath in pairs(modpaths) do
+		pkgmgr.get_mods(modpath, key, global_mods)
 	end
 
 	for i=1,#global_mods,1 do
 		global_mods[i].type = "mod"
 		global_mods[i].loc = "global"
+		global_mods[i].enabled = false
 		retval[#retval + 1] = global_mods[i]
 	end
 
@@ -671,7 +624,7 @@ function pkgmgr.preparemodlist(data)
 		retval[#retval + 1] = {
 			type = "game",
 			is_game_content = true,
-			name = fgettext("$1 mods", gamespec.name),
+			name = fgettext("$1 mods", gamespec.title),
 			path = gamespec.path
 		}
 	end
@@ -692,22 +645,37 @@ function pkgmgr.preparemodlist(data)
 				DIR_DELIM .. "world.mt"
 
 	local worldfile = Settings(filename)
-
-	for key,value in pairs(worldfile:to_table()) do
+	for key, value in pairs(worldfile:to_table()) do
 		if key:sub(1, 9) == "load_mod_" then
 			key = key:sub(10)
-			local element = nil
-			for i=1,#retval,1 do
+			local mod_found = false
+
+			local fallback_found = false
+			local fallback_mod = nil
+
+			for i=1, #retval do
 				if retval[i].name == key and
-					not retval[i].is_modpack then
-					element = retval[i]
-					break
+						not retval[i].is_modpack then
+					if core.is_yes(value) or retval[i].virtual_path == value then
+						retval[i].enabled = true
+						mod_found = true
+						break
+					elseif fallback_found then
+						-- Only allow fallback if only one mod matches
+						fallback_mod = nil
+					else
+						fallback_found = true
+						fallback_mod = retval[i]
+					end
 				end
 			end
-			if element ~= nil then
-				element.enabled = value ~= "false" and value ~= "nil" and value
-			else
-				core.log("info", "Mod: " .. key .. " " .. dump(value) .. " but not found")
+
+			if not mod_found then
+				if fallback_mod and value:find("/") then
+					fallback_mod.enabled = true
+				else
+					core.log("info", "Mod: " .. key .. " " .. dump(value) .. " but not found")
+				end
 			end
 		end
 	end
@@ -744,35 +712,6 @@ function pkgmgr.comparemod(elem1,elem2)
 	return true
 end
 
---------------------------------------------------------------------------------
-function pkgmgr.mod_exists(basename)
-
-	if pkgmgr.global_mods == nil then
-		pkgmgr.refresh_globals()
-	end
-
-	if pkgmgr.global_mods:raw_index_by_uid(basename) > 0 then
-		return true
-	end
-
-	return false
-end
-
---------------------------------------------------------------------------------
-function pkgmgr.get_global_mod(idx)
-
-	if pkgmgr.global_mods == nil then
-		return nil
-	end
-
-	if idx == nil or idx < 1 or
-		idx > pkgmgr.global_mods:size() then
-		return nil
-	end
-
-	return pkgmgr.global_mods:get_list()[idx]
-end
-
 --------------------------------------------------------------------------------
 function pkgmgr.refresh_globals()
 	local function is_equal(element,uid) --uid match
@@ -788,9 +727,9 @@ end
 
 --------------------------------------------------------------------------------
 function pkgmgr.find_by_gameid(gameid)
-	for i=1,#pkgmgr.games,1 do
-		if pkgmgr.games[i].id == gameid then
-			return pkgmgr.games[i], i
+	for i, game in ipairs(pkgmgr.games) do
+		if game.id == gameid then
+			return game, i
 		end
 	end
 	return nil, nil
@@ -801,49 +740,16 @@ function pkgmgr.get_game_mods(gamespec, retval)
 	if gamespec ~= nil and
 		gamespec.gamemods_path ~= nil and
 		gamespec.gamemods_path ~= "" then
-		get_mods(gamespec.gamemods_path, retval)
+		pkgmgr.get_mods(gamespec.gamemods_path, ("games/%s/mods"):format(gamespec.id), retval)
 	end
 end
 
---------------------------------------------------------------------------------
-function pkgmgr.get_game_modlist(gamespec)
-	local retval = ""
-	local game_mods = {}
-	pkgmgr.get_game_mods(gamespec, game_mods)
-	for i=1,#game_mods,1 do
-		if retval ~= "" then
-			retval = retval..","
-		end
-		retval = retval .. game_mods[i].name
-	end
-	return retval
-end
-
---------------------------------------------------------------------------------
-function pkgmgr.get_game(index)
-	if index > 0 and index <= #pkgmgr.games then
-		return pkgmgr.games[index]
-	end
-
-	return nil
-end
-
 --------------------------------------------------------------------------------
 function pkgmgr.update_gamelist()
 	pkgmgr.games = core.get_games()
-end
-
---------------------------------------------------------------------------------
-function pkgmgr.gamelist()
-	local retval = ""
-	if #pkgmgr.games > 0 then
-		retval = retval .. core.formspec_escape(pkgmgr.games[1].name)
-
-		for i=2,#pkgmgr.games,1 do
-			retval = retval .. "," .. core.formspec_escape(pkgmgr.games[i].name)
-		end
-	end
-	return retval
+	table.sort(pkgmgr.games, function(a, b)
+		return a.title:lower() < b.title:lower()
+	end)
 end
 
 --------------------------------------------------------------------------------
